from kuto.core.api.request import HttpReq
from kuto.core.android.element import AdrElem
from kuto.core.android.driver import AndroidDriver
from kuto.core.ios.element import IosElem
from kuto.core.ios.driver import IosDriver
from kuto.core.web.element import WebElem, FraElem
from kuto.core.web.driver import PlayWrightDriver
from kuto.case import TestCase, Page
from kuto.running.runner import main
from kuto.utils.config import config
from kuto.utils.decorate import *
from kuto.utils.log import logger


__version__ = "0.0.21"
__description__ = "移动、web、接口自动化测试框架"



class QException(Exception):
    def __init__(self, msg=None, screen=None, stacktrace=None):
        self.msg = msg
        self.screen = screen
        self.stacktrace = stacktrace

    def __str__(self):
        _msg = f"Message: {self.msg}\n"
        if self.screen is not None:
            _msg += f"Screenshot: available via screen\n"
        if self.stacktrace is not None:
            stacktrace = "\n".join(self.stacktrace)
            _msg += f"Stacktrace:\n{stacktrace}"
        return _msg


class ElementTypeError(QException):
    """
    Find Element types Error
    """
    pass


class NoSuchElementException(QException):
    """
    Element could not found
    """
    pass


class TimeoutException(QException):
    """
    Thrown when a command does not complete in enough time.
    """
    pass


class DeviceNotFoundException(QException):
    """
    设备未找到
    """
    pass


class ElementNameEmptyException(QException):
    """
    控件名称为空
    """
    pass


class ScreenFailException(QException):
    """
    截图失败
    """
    pass


class PlatformError(QException):
    """
    平台设置错误
    """


class DriverNotFound(QException):
    """
    driver未传入
    """


class NoSuchDriverType(QException):
    """
    不支持的驱动类型
    """


class PkgIsNull(QException):
    """
    包名为空
    """


class LocMethodEmptyException(QException):
    """
    定位方式为空
    """


class HostIsNull(QException):
    """
    域名为空
    """


class LocMethodNotOne(QException):
    """
    定位方式超过一种
    """


class ConnectInfoIsNull(QException):
    """
    device_id、remote_addr必须有一个不为空
    """


class SibRemoteConnectFail(QException):
    """
    sib远程连接失败
    """


# pip install pycryptodome==3.14.1
from Crypto.Cipher import AES
import base64


def aes_encrypt(key, content) -> str:
    """
    aes加密
    @param key: 固定的16位秘钥
    @param content: 明文信息
    @return: 密文信息
    """
    aes = AES.new(str.encode(key), AES.MODE_ECB)
    encode_pwd = str.encode(content.rjust(16, '@'))
    encrypt_str = str(base64.encodebytes(aes.encrypt(encode_pwd)), encoding='utf-8')
    return encrypt_str


def aes_decrypt(key, content) -> str:
    """
    aes解密
    @param key: 加密时传入的16位秘钥
    @param content: 密文信息
    @return: 明文信息
    """
    aes = AES.new(str.encode(key), AES.MODE_ECB)
    decrypt_str = (aes.decrypt(base64.decodebytes(content.encode(encoding='utf-8'))).decode().replace('@', ''))
    return decrypt_str





import os
import ssl


import easyocr

from kuto.utils.log import logger

os.environ["CUDA_VISIBLE_DEVICES"] = "True"
os.environ["KMP_DUPLICATE_LIB_OK"] = "True"

ssl._create_default_https_context = ssl._create_unverified_context


class OCRDiscern:
    """
        通过OCR进行文字识别，获取相应坐标值
    """

    def __init__(self) -> None:
        self.image_path = os.path.join(os.path.abspath('screenshots'), 'SourceImage.png')
        if not os.path.exists(self.image_path):
            raise FileNotFoundError(f"文件: {self.image_path} 不存在")
        self.model = ['ch_sim', 'en']
        self.coordinate_set = self.__ocr_read_wording()
        logger.debug(self.coordinate_set)

    # 进行OCR识别
    def __ocr_read_wording(self):
        # 实例化读取器
        reader = easyocr.Reader(self.model)
        # 读取图像
        result = reader.readtext(self.image_path)
        logger.info(result)
        return result

    # 处理坐标数据
    def __handle_coordinate_data(self, coordinate_set, target_wording):
        search_res_coordinate = None
        logger.debug(coordinate_set)
        logger.debug(target_wording)
        for item in coordinate_set:
            if item[1] == target_wording:
                search_res_coordinate = item
                break
        if search_res_coordinate is None or search_res_coordinate[2] < 0.9:
            logger.warning(
                '没有搜索到元素「{0}」或元素「{0}」置信度过低'.format(target_wording))
            return False
        else:
            logger.info('识别到元素「{}」, 置信度为：{}'.format(
                search_res_coordinate[1], search_res_coordinate[2]))
            coordinates = search_res_coordinate[0]
            logger.info(coordinates)
            x_coordinate = coordinates[0][0] + \
                (coordinates[1][0] - coordinates[0][0]) / 2
            y_coordinate = coordinates[0][1] + \
                (coordinates[2][1] - coordinates[1][1]) / 2
            logger.info("X坐标：{}，Y坐标：{}".format(
                x_coordinate, y_coordinate))
            return x_coordinate, y_coordinate

    # 获取坐标信息
    def get_coordinate(self, target_wording: str):
        """get_coordinate 获取指定文字的坐标
        Args:
            target_wording (str): 目标文字
        Returns:
            tuple: x轴坐标 & y轴坐标
        """
        logger.info('开始进行OCR识别🔍')
        res = self.__handle_coordinate_data(self.coordinate_set, target_wording)
        logger.info(res)
        return res


if __name__ == '__main__':
    import time

    start_time = time.time()
    OCRDiscern().get_coordinate('查老板')
    end_time = time.time()
    print('\n ==== OCR cost time: {} ===='.format(end_time - start_time))






# @Time    : 2022/2/22 9:35
# @Author  : kang.yang@qizhidao.com
# @File    : request.py
import json as json_util
import logging
import re

import jmespath
from jsonschema import validate, ValidationError
from requests.packages import urllib3
from urllib import parse

import requests

from kuto.utils.config import config
from kuto.utils.log import logger

# 去掉requests本身的日志
urllib3_logger = logging.getLogger("urllib3")
urllib3_logger.setLevel(logging.CRITICAL)

# 去掉不设置证书的报警
urllib3.disable_warnings()


def formatting(msg):
    """formatted message"""
    if isinstance(msg, dict):
        return json_util.dumps(msg, indent=2, ensure_ascii=False)
    return msg


def request(func):
    def wrapper(*args, **kwargs):
        logger.info("-------------- Request -----------------[🚀]")
        # 给接口带上默认域名
        # 从配置文件中读取域名
        host = config.get_api("base_url")
        # 如果接口路径不以http开头，把域名写到key为url的位置参数中或者第一个参数中
        if "url" in kwargs:
            path: str = kwargs.get("url", "")
            if not path.startswith('http'):
                url = parse.urljoin(host, path)
                kwargs["url"] = url
            else:
                url = path
        else:
            path = list(args)[1]
            if not path.startswith('http'):
                url = parse.urljoin(host, path)
                args_list = list(args)
                args_list[1] = url
                args = tuple(args_list)
            else:
                url = path

        # 请求头处理，写入登录态
        # 从配置文件获取登录用户和游客的请求头
        if kwargs.get("login", True):
            login_header: dict = config.get_api("login")
        else:
            login_header: dict = config.get_api("visit")
        # 把用例脚本中设置的请求头加进来
        header_user_set = kwargs.pop("headers", {})
        login_header.update(header_user_set)
        # 把组装好的请求头装回到kwargs当中
        kwargs["headers"] = login_header

        # 更新超时时间
        timeout_user_set = kwargs.pop("timeout", None)  # 用例脚本中设置的超时时间
        kwargs["timeout"] = timeout_user_set if timeout_user_set else 10

        # 发送请求
        r = func(*args, **kwargs)

        # 输出请求参数日志
        logger.debug("[method]: {m}      [url]: {u}".format(m=func.__name__.upper(), u=url))
        auth = kwargs.get("auth", "")
        if auth:
            logger.debug(f"[auth]:\n {formatting(auth)}")
        logger.debug(f"[headers]:\n {formatting(dict(r.request.headers))}")
        cookies = kwargs.get("cookies", "")
        if cookies:
            logger.debug(f"[cookies]:\n {formatting(cookies)}")
        params = kwargs.get("params", "")
        if params:
            logger.debug(f"[params]:\n {formatting(params)}")
        data = kwargs.get("data", "")
        if data:
            logger.debug(f"[data]:\n {formatting(data)}")
        json = kwargs.get("json", "")
        if json:
            logger.debug(f"[json]:\n {formatting(json)}")

        # 保存响应结果并输出日志
        status_code = r.status_code
        headers = r.headers
        content_type = headers.get("Content-Type")
        ResponseResult.status_code = status_code
        logger.info("-------------- Response ----------------")
        logger.debug(f"[status]: {status_code}")
        logger.debug(f"[headers]: {formatting(headers)}")
        try:
            resp = r.json()
            logger.debug(f"[type]: json")
            logger.debug(f"[response]:\n {formatting(resp)}")
            ResponseResult.response = resp
        except Exception:
            # 非json响应数据，根据响应内容类型进行判断
            if content_type is not None:
                if "text" not in content_type:
                    logger.debug(f"[type]: {content_type}")
                    logger.debug(f"[response]:\n {r.content}")
                    ResponseResult.response = r.content
                else:
                    logger.debug(f"[type]: {content_type}")
                    logger.debug(f"[response]:\n {r.text}")
                    ResponseResult.response = r.text
            else:
                logger.debug('ContentType为空，响应异常！！！')
                ResponseResult.response = r.text

    return wrapper


class ResponseResult:
    status_code = 200
    response = None


class HttpReq(object):
    @request
    def get(self, url, params=None, verify=False, login=True, **kwargs):
        return requests.get(url, params=params, verify=verify, **kwargs)

    @request
    def post(self, url, data=None, json=None, verify=False, login=True, **kwargs):
        return requests.post(url, data=data, json=json, verify=verify, **kwargs)

    @request
    def put(self, url, data=None, json=None, verify=False, login=True, **kwargs):
        if json is not None:
            data = json_util.dumps(json)
        return requests.put(url, data=data, verify=verify, **kwargs)

    @request
    def delete(self, url, verify=False, login=True, **kwargs):
        return requests.delete(url, verify=verify, **kwargs)

    @property
    def response(self):
        """
        Returns the result of the response
        :return: response
        """
        return ResponseResult.response

    # 断言
    @staticmethod
    def assertStatusCode(status_code):
        """
        兼容历史版本
        """
        actual_code = ResponseResult.status_code
        logger.info(f"断言: {actual_code} 等于 {status_code}")
        assert (
                actual_code == status_code
        ), f"status_code {ResponseResult} != {status_code}"

    @staticmethod
    def assertSchema(schema, response=None) -> None:
        """
        Assert JSON Schema
        doc: https://json-schema.org/
        """
        logger.info(f"assertSchema -> {formatting(schema)}.")

        if response is None:
            response = ResponseResult.response

        try:
            validate(instance=response, schema=schema)
        except ValidationError as msg:
            assert "Response data" == "Schema data", msg

    @staticmethod
    def assertEq(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 等于 {value}")
        assert search_value == value, f"{search_value} != {value}"

    @staticmethod
    def assertNotEq(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 不等于 {value}")
        assert search_value != value, f"{search_value} 等于 {value}"

    @staticmethod
    def assertLenEq(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {len(search_value)} 等于 {value}")
        assert len(search_value) == value, f"{search_value} 的长度不等于 {value}"

    @staticmethod
    def assertLenGt(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {len(search_value)} 大于 {value}")
        assert len(search_value) > value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenGtOrEq(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {len(search_value)} 大于等于 {value}")
        assert len(search_value) >= value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenLt(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {len(search_value)} 小于 {value}")
        assert len(search_value) < value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenLtOrEq(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {len(search_value)} 小于等于 {value}")
        assert len(search_value) <= value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertGt(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            if "." in search_value:
                search_value = float(search_value)
            else:
                search_value = int(search_value)
        logger.info(f"断言: {search_value} 大于 {value}")
        assert search_value > value, f"{search_value} 不大于 {value}"

    @staticmethod
    def assertGtOrEq(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            if "." in search_value:
                search_value = float(search_value)
            else:
                search_value = int(search_value)
        logger.info(f"断言: {search_value} 大于等于 {value}")
        assert search_value >= value, f"{search_value} 小于 {value}"

    @staticmethod
    def assertLt(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            if "." in search_value:
                search_value = float(search_value)
            else:
                search_value = int(search_value)
        logger.info(f"断言: {search_value} 小于 {value}")
        assert search_value < value, f"{search_value} 不大于 {value}"

    @staticmethod
    def assertLtOrEq(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            if "." in search_value:
                search_value = float(search_value)
            else:
                search_value = int(search_value)
        logger.info(f"断言: {search_value} 小于等于 {value}")
        assert search_value <= value, f"{search_value} 不大于 {value}"

    @staticmethod
    def assertRange(path, start, end):
        """兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            if "." in search_value:
                search_value = float(search_value)
            else:
                search_value = int(search_value)
        logger.info(f"断言: {search_value} 在 [{start}, {end}] 范围内")
        assert (search_value >= start) & (
                search_value <= end
        ), f"{search_value} 不在[{start}, {end}]范围内"

    @staticmethod
    def assertIn(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 被 {value} 包含")
        assert search_value in value, f"{value} 不包含 {search_value}"

    @staticmethod
    def assertNotIn(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 不被 {value} 包含")
        assert search_value not in value, f"{value} 包含 {search_value}"

    @staticmethod
    def assertNotExists(path):
        """兼容历史版本"""
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {path} 不存在或值为None")
        assert search_value is None, f"仍然包含 {path} 为 {search_value}"

    @staticmethod
    def assertContain(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 包含 {value}")
        assert value in search_value, f"{search_value} 不包含 {value}"

    @staticmethod
    def assertNotContain(path, value):
        """
        兼容历史版本
        """
        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 不包含 {value}")
        assert value not in search_value, f"{search_value} 包含 {value}"

    @staticmethod
    def assertTypeMatch(path, value_type):
        """
        兼容历史版本
        """
        if not isinstance(value_type, type):
            if value_type == "int":
                value_type = int
            elif value_type == "str":
                value_type = str
            elif value_type == "list":
                value_type = list
            elif value_type == "dict":
                value_type = dict
            else:
                value_type = str

        search_value = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 是 {value_type} 类型")
        assert isinstance(
            search_value, value_type
        ), f"{search_value} 不是 {value_type} 类型"

    @staticmethod
    def assertStartsWith(path, value):
        """
        兼容历史版本
        """
        search_value: str = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 以 {value} 开头")
        assert search_value.startswith(value), f"{search_value} 不以 {value} 开头"

    @staticmethod
    def assertEndsWith(path, value):
        """
        兼容历史版本
        """
        search_value: str = jmespath.search(path, ResponseResult.response)
        logger.info(f"断言: {search_value} 以 {value} 结尾")
        assert search_value.endswith(value), f"{search_value} 不以 {value} 结尾"

    @staticmethod
    def assertRegexMatch(path, value):
        """同assert_regex_match，兼容历史版本"""
        search_value = jmespath.search(path, ResponseResult.response)
        match_obj = re.match(r"" + value, search_value, flags=re.I)
        logger.info(f"断言: {search_value} 匹配正则表达式 {value} 成功")
        assert match_obj is not None, f"结果 {search_value} 匹配失败"





"""
使用allure需要下载，并配置到环境变量，下载地址：https://github.com/allure-framework/allure2/releases
"""
import os
# import allure
import pytest
import inspect as sys_inspect

from kuto.utils.log import logger
from kuto.utils.read_file import read_json, read_yaml, read_csv, read_excel


# # 对应模块
# def epic(text):
#     return allure.epic(text)
#
#
# # 对应class
# def feature(text):
#     return allure.feature(text)
#
#
# # 对应method
# def story(text):
#     return allure.story(text)
#
#
# # 报告子模块，兼容历史版本
# def module(text):
#     return allure.story(text)
#
#
# # 用例标题标题
# def title(text):
#     return allure.title(text)
#
#
# # 报告步骤
# def step(text):
#     return allure.step(text)


def order(index):
    """
    指定用例执行顺序
    doc: https://blog.csdn.net/weixin_43880991/article/details/116221362
    """
    return pytest.mark.run(order=index)


def depend(depends: list or str = None, name=None):
    """
    设置用例依赖关系
    doc: https://www.cnblogs.com/se7enjean/p/13513131.html
    """
    if isinstance(depends, str):
        depends = [depends]
    return pytest.mark.dependency(name=name, depends=depends)


# 参数化数据
def data(list_data: list):
    """
    必须传入一个list，使用时通过在参数列表传入parma进行调用
    """
    return pytest.mark.parametrize('param', list_data)


# 从json文件获取数据进行参数化
def file_data(file=None, key=None, row=None):
    # logger.debug(config.get_env())

    """
    @param row: 第几行，针对csv和excel文件
    @param file: 文件名
    @param key: 针对json和yaml文件
    """
    # 获取被装饰方法的目录
    stack_t = sys_inspect.stack()
    ins = sys_inspect.getframeinfo(stack_t[1][0])
    file_dir = os.path.dirname(os.path.abspath(ins.filename))
    # logger.debug(file_dir)
    parent_dir = os.path.dirname(file_dir)
    parent_dir_dir = os.path.dirname(parent_dir)
    parent_dir_dir_dir = os.path.dirname(parent_dir_dir)

    # 判断当前、父目录、爷目录、太爷爷目录下的test_data目录下是否有file_name文件
    path_list = [file_dir, parent_dir, parent_dir_dir, parent_dir_dir_dir]
    file_path = None
    full_list = []
    for _path in path_list:
        # environ = config.get_env()
        # if environ:
        #     full_path = os.path.join(_path, 'data', environ, file)
        # else:
        full_path = os.path.join(_path, 'data', file)
        full_list.append(full_path)
        # print(full_path)
        if os.path.isfile(full_path) is True:
            file_path = full_path
            break
    if file_path is None:
        raise Exception(f"can not found {file} in {full_list}")

    # logger.debug(file_path)
    if file_path.endswith(".json"):
        content = read_json(file_path, key)
    elif file_path.endswith(".yml"):
        content = read_yaml(file_path, key)
    elif file_path.endswith(".csv"):
        content = read_csv(file_path, row)
    elif file_path.endswith(".xlsx"):
        content = read_excel(file_path, row)
    else:
        raise TypeError("不支持的文件类型，仅支持json、yml、csv、xlsx")

    if content:
        return data(content)
    else:
        raise ValueError('数据不能为空')




import os

from kuto.utils.exceptions import DeviceNotFoundException


def get_device_list():
    """获取当前连接的手机列表"""
    cmd = 'adb devices'
    output = os.popen(cmd).read()
    device_list = [item.split('\t')[0] for item in output.split('\n') if item.endswith('device')]
    if len(device_list) > 0:
        return device_list
    else:
        raise DeviceNotFoundException(msg=f"无已连接设备")


def get_current_device():
    """连接一个手机时，返回设备id"""
    device_list = get_device_list()
    return device_list[0]


def check_device(device_id):
    """检查设备是否已连接"""
    if device_id is None:
        device_id = get_current_device()
        return device_id
    else:
        if device_id in get_device_list():
            return device_id
        else:
            raise DeviceNotFoundException(msg=f"设备 {device_id} 未连接")




import json
import time
from typing import Union
from urllib import parse

from kuto.core.android.driver import AndroidDriver
from kuto.core.api.request import HttpReq
from kuto.core.ios.driver import IosDriver
from kuto.utils.config import config
from kuto.utils.log import logger
from kuto.utils.exceptions import (
    HostIsNull
)
from kuto.core.web.driver import PlayWrightDriver
from kuto.core.web.element import WebElem


class Page(object):
    """页面基类，用于pom模式封装"""

    def __init__(self, driver: Union[AndroidDriver, IosDriver]):
        self.driver = driver

    @staticmethod
    def sleep(n):
        """休眠"""
        logger.info(f"休眠 {n} 秒")
        time.sleep(n)


class TestCase(HttpReq):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    driver: Union[AndroidDriver, IosDriver, PlayWrightDriver] = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.start_time = time.time()
        platform = config.get_common("platform")
        if platform == "android":
            device_id = config.get_app("device_id")
            pkg_name = config.get_app("pkg_name")
            self.driver = AndroidDriver(device_id, pkg_name)
        elif platform == "ios":
            device_id = config.get_app("device_id")
            pkg_name = config.get_app("pkg_name")
            self.driver = IosDriver(device_id, pkg_name)
        elif platform == "web":
            browserName = config.get_web("browser_name")
            headless = config.get_web("headless")
            state = config.get_web("state")
            if state:
                state_json = json.loads(state)
                self.driver = PlayWrightDriver(browserName=browserName, headless=headless, state=state_json)
            else:
                self.driver = PlayWrightDriver(browserName=browserName, headless=headless)
        if isinstance(self.driver, (AndroidDriver, IosDriver)):
            self.driver.start_app()
        self.start()

    def teardown_method(self):
        self.end()
        if isinstance(self.driver, PlayWrightDriver) == "web":
            self.driver.close()
        if isinstance(self.driver, (AndroidDriver, IosDriver)):
            self.driver.stop_app()
        take_time = time.time() - self.start_time
        logger.debug("[run_time]: {:.2f} s".format(take_time))

    @staticmethod
    def sleep(n: float):
        """休眠"""
        logger.debug(f"sleep: {n}s")
        time.sleep(n)

    def open(self, url=None):
        """浏览器打开页面"""
        # 拼接域名
        if url is None:
            base_url = config.get_web("base_url")
            if not base_url:
                raise HostIsNull('base_url is null')
            url = base_url
        else:
            if "http" not in url:
                base_url = config.get_web("base_url")
                if not base_url:
                    raise HostIsNull('base_url is null')
                url = parse.urljoin(base_url, url)
        # 访问页面
        self.driver.visit(url)
        # 设置cookies
        cookies = config.get_web("cookies")
        if cookies:
            self.driver.set_cookies(cookies)

    def new_page(self, locator: WebElem):
        """切换到新页签"""
        self.driver.switch_tab(locator)

    def screenshot(self, name: str):
        """截图"""
        self.driver.screenshot(name)




import kuto
from kuto import IosElem


class IndexPage(kuto.Page):
    """首页"""
    ad = IosElem(text='close white big')
    my = IosElem(text='我的')


class MyPage(kuto.Page):
    """我的页"""
    setting = IosElem(text='settings navi')


class TestSearch(kuto.TestCase):
    """设置页测试"""

    def start(self):
        self.index_page = IndexPage(self.driver)
        self.my_page = MyPage(self.driver)

    def test_1(self):
        """进入设置页"""
        self.index_page.ad.click_exists(timeout=5)
        self.index_page.my.click()
        self.my_page.setting.click()


if __name__ == '__main__':
    kuto.main(did='00008101-000E646A3C29003A', pkg='com.qizhidao.company')




import os
import yaml


local_path = os.path.dirname(os.path.realpath(__file__))
root_path = os.path.dirname(local_path)


class Config:
    def __init__(self):
        self.file_path = os.path.join(root_path, 'running', 'conf.yml')

    def get(self, module, key):
        try:
            with open(self.file_path, "r", encoding="utf-8") as f:
                yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
            return yaml_data[module][key]
        except Exception as e:
            print(e)
            return None

    def get_api(self, key):
        return self.get('api', key)

    def get_app(self, key):
        return self.get('app', key)

    def get_web(self, key):
        return self.get('web', key)

    def get_common(self, key):
        return self.get('common', key)

    def set(self, module, key, value):
        try:
            with open(self.file_path, "r", encoding="utf-8") as f:
                yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)
            yaml_data[module][key] = value
            with open(self.file_path, 'w', encoding="utf-8") as f:
                yaml.dump(yaml_data, f)
        except Exception as e:
            print(e)

    def set_api(self, key, value):
        self.set('api', key, value)

    def set_app(self, key, value):
        self.set('app', key, value)

    def set_web(self, key, value):
        self.set('web', key, value)

    def set_common(self, key, value):
        self.set('common', key, value)


config = Config()


if __name__ == '__main__':
    print(config.get('app', 'device_id'))









import kuto
from kuto import data, file_data


LIST_DATA = [
    {"name": "李雷", "age": "33"},
    {"name": "韩梅梅", "age": "30"}
]


class TestParameter(kuto.TestCase):
    """
    原则是无论是哪种方式，返回的数据必须是list，用例都通过"param"进行调用
    """

    @data(LIST_DATA)
    def test_list(self, param):
        print(param)

    @file_data(file='data.json')
    def test_json(self, param):
        print(param)

    @file_data(file='data.yml', key='names')
    def test_yaml(self, param):
        print(param)

    @file_data(file='data.csv')
    def test_csv(self, param):
        print(param)

    @file_data(file='data.xlsx', row=1)
    def test_excel(self, param):
        print(param)


if __name__ == '__main__':
    kuto.main()



import os
import subprocess

from kuto.utils.log import logger
from kuto.utils.exceptions import DeviceNotFoundException


def get_device_list():
    """获取当前连接的设备列表"""
    cmd = 'tidevice list'
    output = os.popen(cmd).read()
    device_list = [item.split(' ')[0] for item in output.split('\n') if item]
    if len(device_list) > 0:
        return device_list
    else:
        raise DeviceNotFoundException(msg=f"无已连接设备")


def get_current_device():
    """连接一个手机时，返回设备id"""
    device_list = get_device_list()
    return device_list[0]


def get_tcp_port(udid: str):
    """获取可用端口号"""
    port = int(udid.split('-')[0])
    return port


def check_device(device_id):
    """检查设备是否已连接"""
    if device_id is None:
        device_id = get_current_device()
        return device_id
    else:
        if device_id in get_device_list():
            return device_id
        else:
            raise DeviceNotFoundException(msg=f"设备 {device_id} 未连接")


class TideviceUtil:
    """
    tidevice常用功能的封装
    """

    @staticmethod
    def uninstall_app(device_id=None, pkg_name=None):
        """卸载应用"""
        cmd = f"tidevice -u {device_id} uninstall {pkg_name}"
        logger.info(f"卸载应用: {pkg_name}")
        output = subprocess.getoutput(cmd)
        if "Complete" in output.split()[-1]:
            logger.info(f"{device_id} 卸载应用{pkg_name} 成功")
            return
        else:
            logger.info(f"{device_id} 卸载应用{pkg_name}失败，因为{output}")

    @staticmethod
    def install_app(device_id=None, ipa_url=None):
        """安装应用
        """
        cmd = f"tidevice -u {device_id} install {ipa_url}"
        logger.info(f"安装应用: {ipa_url}")
        output = subprocess.getoutput(cmd)
        if "Complete" in output.split()[-1]:
            logger.info(f"{device_id} 安装应用{ipa_url} 成功")
            return
        else:
            logger.info(f"{device_id} 安装应用{ipa_url}失败，因为{output}")


if __name__ == '__main__':
    print(get_device_list())




"""
genson:
https://github.com/wolverdude/GenSON
"""
import json
# from genson import SchemaBuilder
# from kuto.utils.log import logger


def formatting(msg):
    """formatted message"""
    if isinstance(msg, dict):
        return json.dumps(msg, indent=2, ensure_ascii=False)
    return msg


# def genSchema(data: dict = None):
#     """
#     return schema data
#     """
#     builder = SchemaBuilder()
#     builder.add_object(data)
#     to_schema = builder.to_schema()
#     to_schema.pop('$schema')
#     logger.debug(f'生成schmea如下:\nschema start-------------->\n{formatting(to_schema)}\nschema end-------------->')
#     return to_schema




"""
@Author: kang.yang
@Date: 2023/5/9 10:02
"""
import allure


def upload_pic(file_path: str):
    """上传图片到allure报告"""
    print("上传图片到allure报告")
    file_name = "截图"
    print(file_name)
    allure.attach.file(
        file_path,
        attachment_type=allure.attachment_type.PNG,
        name={file_name}
    )



import os
import shutil
import subprocess
import time

# import allure
import allure
import wda

from kuto.core.ios.common import get_tcp_port, check_device
from kuto.utils.exceptions import ScreenFailException, PkgIsNull, DeviceNotFoundException
from kuto.utils.log import logger
from kuto.core.ios.common import TideviceUtil
from kuto.testdata import get_int


def _start_wda_xctest(udid: str, port, wda_bundle_id=None) -> bool:
    xctool_path = shutil.which("tidevice")
    logger.info(f"WDA is not running, exec: {xctool_path} -u {udid} wdaproxy --port {port} -B {wda_bundle_id}")
    args = []
    if udid:
        args.extend(["-u", udid])
    args.append("wdaproxy")
    args.extend(["--port", str(port)])
    if wda_bundle_id is not None:
        args.extend(["-B", wda_bundle_id])
    p = subprocess.Popen([xctool_path] + args)
    time.sleep(3)
    if p.poll() is not None:
        logger.warning("xctest launch failed")
        return False
    return True


class IosDriver(object):

    def __init__(self, device_id=None, pkg_name=None):
        logger.info(f'init ios driver.')

        self.pkg_name = pkg_name
        if not self.pkg_name:
            raise PkgIsNull('pkg is null')
        self.device_id = device_id
        if not self.device_id:
            raise DeviceNotFoundException('deviceId is null')

        self.device_id = check_device(self.device_id)  # 检查本地设备是否已连接
        self.port = get_tcp_port(self.device_id)
        self.wda_url = f"http://localhost:{self.port}"
        self.d = wda.Client(self.wda_url)

        # check if wda is ready
        if self.d.is_ready():
            logger.info('wda is ready.')
        else:
            logger.info('wda is not ready, start now.')
            _start_wda_xctest(self.device_id, port=self.port)


    @property
    def device_info(self):
        """设备信息"""
        info = self.d.device_info()
        logger.info(f"get device info: {info}")
        return info

    @property
    def page_content(self):
        """获取页面xml内容"""
        logger.info('get page content')
        page_source = self.d.source(accessible=False)
        return page_source

    def install_app(self, ipa_url, new=True, pkg_name=None):
        """安装应用
        @param ipa_url: ipa链接
        @param new: 是否先卸载
        @param pkg_name: 应用包名
        @return:
        """
        logger.info(f"install: {ipa_url}")
        if new is True:
            pkg_name = pkg_name if pkg_name else self.pkg_name
            self.uninstall_app(pkg_name)

        TideviceUtil.install_app(self.device_id, ipa_url)

    def uninstall_app(self, pkg_name=None):
        """卸载应用"""
        logger.info(f"uninstall: {pkg_name}")
        pkg_name = pkg_name if pkg_name else self.pkg_name

        TideviceUtil.uninstall_app(self.device_id, pkg_name)

    def start_app(self, pkg_name=None, stop=True):
        """启动应用
        @param pkg_name: 应用包名
        @param stop: 是否先停止应用
        """
        logger.info(f"start app: {pkg_name}")
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"启动应用: {pkg_name}")
        if stop is True:
            self.d.app_terminate(pkg_name)
        self.d.app_start(pkg_name)

    def stop_app(self, pkg_name=None):
        """停止应用"""
        logger.info(f"stop app")
        if not pkg_name:
            pkg_name = self.pkg_name
        logger.info(f"停止应用: {pkg_name}")
        self.d.app_terminate(pkg_name)

    def back(self):
        """返回上一页"""
        logger.info("back")
        time.sleep(1)
        self.d.swipe(0, 100, 100, 100)

    def set_text(self, value):
        """输入内容"""
        logger.info(f"input: {value}")
        self.d.send_keys(value)

    def screenshot(self, file_name=None, with_time=True):
        """
        截图并保存到预定路径
        @param with_time: 是否带时间戳
        @param file_name: foo.png or fool
        @return:
        """
        if not file_name:
            raise ValueError("file_name should not be None.")

        try:
            # 把文件名处理成test.png的样式
            if "." in file_name:
                file_name = file_name.split(r".")[0]
            # 截图并保存到当前目录的images文件夹中
            relative_path = "screenshots"
            if os.path.exists(relative_path) is False:
                os.mkdir(relative_path)
            if with_time:
                time_str = time.strftime(f"%Y-%m-%d_%H_%M_%S_{get_int(min_size=1, max_size=1000)}")
                file_name = f"{time_str}_{file_name}.png"
            else:
                file_name = f'{file_name}.png'

            file_path = os.path.join(relative_path, file_name)
            logger.info(f"save to: {os.path.join(relative_path, file_name)}")
            self.d.screenshot(file_path)
            # 上传allure报告
            allure.attach.file(
                file_path,
                attachment_type=allure.attachment_type.PNG,
                name=f"{file_name}",
            )
            return file_path
        except Exception as e:
            raise ScreenFailException(f"screenshot fail: \n{str(e)}")

    def click(self, x, y):
        """点击坐标"""
        logger.info(f"click: ({x}, {y})")
        logger.info(f"{self.device_id} Tap point ({x}, {y})")
        self.d.appium_settings({"snapshotMaxDepth": 0})
        self.d.tap(x, y)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(1)

    def click_alerts(self, alert_list: list):
        """点击弹窗"""
        try:
            self.d.alert.click(alert_list)
        except:
            pass

    def swipe(self, start_x, start_y, end_x, end_y, duration=0):
        """根据坐标滑动"""
        logger.info(f"from ({start_x}, {start_y}) swipe to ({end_x}, {end_y})")
        logger.info(
            f"{self.device_id} swipe from point ({start_x}, {start_y}) to ({end_x}, {end_y})"
        )
        self.d.appium_settings({"snapshotMaxDepth": 2})
        self.d.swipe(int(start_x), int(start_y), int(end_x), int(end_y), duration)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(2)

    def swipe_left(self, start_percent=1, end_percent=0.5):
        """往左滑动"""
        logger.info("swipe left")
        w, h = self.d.window_size()
        self.swipe(start_percent * (w - 1), h / 2, end_percent * w, h / 2)

    def swipe_right(self, start_percent=0.5, end_percent=1):
        """往右滑动"""
        logger.info("swipe right")
        w, h = self.d.window_size()
        self.swipe(start_percent * w, h / 2, end_percent * (w - 1), h / 2)

    def swipe_up(self, start_percent=0.8, end_percent=0.2):
        """往上滑动"""
        logger.info("swipe up")
        w, h = self.d.window_size()
        self.swipe(w / 2, start_percent * h, w / 2, end_percent * h)

    def swipe_down(self, start_percent=0.2, end_percent=0.8):
        """往下滑动"""
        logger.info("swipe down")
        w, h = self.d.window_size()
        self.swipe(w / 2, start_percent * h, w / 2, end_percent * h)

    def health_check(self):
        """检查设备连接状态"""
        logger.info("health check")
        self.d.healthcheck()

    def open_url(self, url):
        """
        打开schema
        @param: url，schema链接，taobao://m.taobao.com/index.htm
        @return:
        """
        logger.info(f"open url or schema: {url}")
        self.d.open_url(url)


if __name__ == '__main__':
    driver = IosDriver('00008101-000E646A3C29003A', 'com.qizhidao.company')
    driver.install_app('http://172.16.5.225:8081/AppUpdateTest/pre_qizhiyun_client_4.3.2.ipa')






import kuto


if __name__ == '__main__':
    # 主程序入口

    # 执行接口用例
    kuto.main(
        host='https://app-pre.qizhidao.com',
        path='tests/test_api.py'
    )

    # # 执行安卓用例
    # kuto.main(
    #     did='UJK0220521066836',
    #     pkg='com.qizhidao.clientapp',
    #     path='tests/test_adr.py'
    # )
    #
    # # 执行ios用例
    # kuto.main(
    #     did='00008101-000E646A3C29003A',
    #     pkg='com.qizhidao.company',
    #     path='tests/test_ios.py'
    # )
    #
    # # 执行web用例
    # kuto.main(
    #     browser="chrome",
    #     host='https://www.qizhidao.com/',
    #     path='tests/test_web.py',
    # )



import json
import os.path


class AllureData:
    """解析allure_results的数据"""

    def __init__(self, result_path, product=None, test_type=None):
        self.result_path = result_path
        self.product = product
        self.test_type = test_type

    def get_files(self):
        """获取以result.json结尾的文件列表"""
        file_list = []
        for filename in os.listdir(self.result_path):
            if filename.endswith('result.json'):
                file_list.append(filename)
        return [os.path.join(self.result_path, item) for item in os.listdir(self.result_path) if
                item.endswith('result.json')]

    @staticmethod
    def get_file_content(file_path):
        """获取文件内容并转成json"""
        with open(file_path, 'r', encoding='UTF-8') as f:
            content = json.load(f)
        return content

    def parser_content(self, content):
        """解析执行结果"""
        name = content.get('name')
        status = content.get('status')
        full_name = content.get('fullName')
        decription = content.get('description')
        parameters = content.get('parameters', None)
        log_name = [os.path.join(self.result_path, item.get('source')) for item in content.get('attachments') if
                    item.get('name') == 'log'][0]
        start = content.get('start')
        end = content.get('stop')
        case_data = {
            "name": name,
            "full_name": full_name,
            "description": decription,
            "log": log_name,
            "status": status,
            "start_time": start,
            "end_time": end,
            "parameters": parameters
        }
        return case_data

    @staticmethod
    def remove_duplicate(json_contents):
        """去除失败重试的重复结果"""
        case_list = []
        no_repeat_tags = []
        for item in json_contents:
            full_name = item["full_name"]
            parameters = item["parameters"]
            if (full_name, parameters) not in no_repeat_tags:
                no_repeat_tags.append((full_name, parameters))
                case_list.append(item)
            else:
                for case in case_list:
                    if case.get('full_name') == full_name and case.get('parameters') == parameters:
                        if case.get('status') != 'passed':
                            case_list.remove(case)
                            case_list.append(item)
        return case_list

    def get_results(self):
        """获取对外的测试结果列表"""
        file_list = self.get_files()
        result_list = []
        for file in file_list:
            content = self.get_file_content(file)
            parser_content = self.parser_content(content)
            result_list.append(parser_content)
        return self.remove_duplicate(result_list)

    def get_statistical_data(self):
        case_list = self.get_results()

        # 获取用例统计数据
        passed_list = []
        fail_list = []
        for case in case_list:
            status = case.get('status')
            if status == 'passed':
                passed_list.append(case)
            else:
                fail_list.append(case)
        total = len(case_list)
        passed = len(passed_list)
        failed = len(fail_list)
        rate = round((passed / total) * 100, 2)

        # 获取整个任务的开始和结束时间
        start_time, end_time = case_list[0].get('start_time'), case_list[0].get('end_time')
        for case in case_list:
            inner_start = case.get('start_time')
            inner_end = case.get('end_time')
            if inner_start < start_time:
                start_time = inner_start
            if inner_end > end_time:
                end_time = inner_end

        return {
            'total': total,
            'passed': passed,
            'failed': failed,
            'rate': rate,
            'start': start_time,
            'end': end_time,
            'product': self.product,
            'type': self.test_type
        }


def get_allure_data(result_path):
    """兼容邮件和钉钉的调用"""
    return AllureData(result_path).get_statistical_data()


class ApiData(AllureData):
    """接口测试统计数据"""
    def __init__(self, result_path, product=None):
        super(ApiData, self).__init__(result_path, product, 'api')

    def get_interfaces(self):
        case_list = self.get_results()
        interface_list = []
        for case in case_list:
            log = case.get('log')
            with open(log, 'r') as f:
                for line in f.readlines():
                    if 'url]: ' in line:
                        interface = line.strip().split('url]: ')[1]
                        interface_list.append(interface)
        interface_list = list(set(interface_list))
        return interface_list

    def get_statistic_data(self):
        statistic_data: dict = self.get_statistical_data()
        interface_count = len(self.get_interfaces())
        statistic_data["num"] = interface_count
        return statistic_data


class AppData(AllureData):
    """app测试统计数据"""

    def __init__(self, result_path, product=None):
        super(AppData, self).__init__(result_path, product, 'app')

    def get_statistic_data(self):
        return self.get_statistical_data()


class WebData(AllureData):
    """web页面测试统计数据"""
    def __init__(self, result_path, product=None):
        super(WebData, self).__init__(result_path, product, 'web')

    def get_statistic_data(self):
        return self.get_statistical_data()


if __name__ == '__main__':
    result_path1 = '/Users/UI/PycharmProjects/qrunner_new_gitee/reports'
    print(get_allure_data(result_path1))

